
open Cil
(* open Utils *)
open FreeSafeTyUtil
open CilPretty
(* open ContainsLock *)
open FreeSafeTyFlag

let dumpId = ref 0

let dump f name =
  (* if !dumpIntermFile then *)
    let _ = incr dumpId in
    let filename = name in
    let fout = open_out ("fstDebug_" ^ filename) in
      dumpFile defaultCilPrinter fout filename f;
      close_out fout

(*
let rec sort_globals_in_file f =
  f.globals <- sort_globals f.globals
and sort_globals gs =
  List.sort compare_global gs
and compare_global g1 g2 =
  match g1, g2 with
    | (GType _ | GCompTag _ | GCompTagDecl _ | GEnumTag _ | GEnumTagDecl _), 
      (GType _ | GCompTag _ | GCompTagDecl _ | GEnumTag _ | GEnumTagDecl _) -> 0
    | (GVarDecl _ | GVar _ | GFun _ | GAsm _ | GPragma _ | GText _), 
	(GVarDecl _ | GVar _ | GFun _ | GAsm _ | GPragma _ | GText _) -> 0 
    | (GType _ | GCompTag _ | GCompTagDecl _ | GEnumTag _ | GEnumTagDecl _), _ -> -1
    | _, (GType _ | GCompTag _ | GCompTagDecl _ | GEnumTag _ | GEnumTagDecl _) -> 1
*)

let feature : featureDescr = 
  { fd_name = "freesafety";
    fd_enabled = ref true;
    fd_description = "Perform memory usage analysis";
    fd_extraopt = [];
    fd_doit = 
      (function (f: file) ->
	 lineDirectiveStyle := Some(LineComment);
	 let _ = FreeSafeTyFlag.currentFileName := f.fileName in
	 let _ = Format.printf "Inspecting %s...@\n" f.fileName in
	 let tuneCallInstrVisitor = new TuneFunctionType.tuneCallInstrVisitor in
	 let tuneFunctionTypeVisitor = new TuneFunctionType.tuneFunctionTypeVisitor in
	 (* let checkNullLvalVisitor = new FreeSafeTyCheckNullLvals.checkNullLvalVisitor in *)
	   (*
	     let pointifyStructVisitor = new TuneStructType.pointifyStructFields in
	   *)
	 let checkAddrOfVisitor = new CheckAddrOf.checkAddrOfVisitor in
	 let normalizeLvalVisitor = new NormalizeLval.normalizeLvalVisitor in
	   if !FreeSafeTyFlag.dumpIntermFile then dump f "beforeProcess";
	   visitCilFile tuneCallInstrVisitor f;
	   if !FreeSafeTyFlag.dumpIntermFile then dump f "callTuned";
	   visitCilFile tuneFunctionTypeVisitor f;
	   (* fstwarn "Funtyp adjustment completed.@\n%!"; *)
	   if !FreeSafeTyFlag.dumpIntermFile then dump f "funtypTuned";
	   (* visitCilFile checkNullLvalVisitor f; *)
	   (*
	     visitCilFile pointifyStructVisitor f;
	     dump f "structPointified";
	   *)
	   visitCilFile checkAddrOfVisitor f;
	   (* fstwarn "Addrof check completed.@\n%!"; *)
	   visitCilFile normalizeLvalVisitor f;
	   (* fstwarn "Lval normalization completed.@\n%!"; *)
	   if !FreeSafeTyFlag.dumpIntermFile then dump f "lvalNormalized";
	   (* visitCilFile setupEnvVisitor f; *)
	   (* Rmtmps.removeUnusedTemps f; *)
	   (* fstwarn "Unused temps are removed.@\n%!"; *)
	   FreeSafeTyGenConstr.setupEnv f;
	   (* fstwarn "Environment is setup.@\n%!"; *)
	   Cfg.computeFileCFG f;
	   (* fstwarn "CFG computed.@\n%!"; *)
	   let te, cs = 
	     if !FreeSafeTyFlag.measureTime then
	       measure_and_print "genconstr" (fun () -> FreeSafeTyGenConstr.f f) () 
	     else 
	       FreeSafeTyGenConstr.f f
	   in
	   let cs = 
	     if !FreeSafeTyFlag.measureTime then
	       measure_and_print "preproc" (fun () -> FreeSafeTyPreprocConstr.f cs) () 
	     else 
	       FreeSafeTyPreprocConstr.f cs
	   in
	   (* let _ = if !FreeSafeTyFlag.debug then Format.printf "<preproced cs>@[%a@]@\n" FreeSafeTyGenConstr.print_cs cs in *)
	   let b = 
	     if !FreeSafeTyFlag.measureTime then
	       measure_and_print "redconstr" (fun () -> FreeSafeTyRedConstr.f te cs) () 
	     else
	       FreeSafeTyRedConstr.f te cs
	   in
	     if not b then failwith "Type error"
      );
    fd_post_check = false
  }
