
open Graph
open Gml
open Graphviz

open Cil
open FreeSafeTyGenConstr
  (* open RedConstr *)
open Glpk
open GlpkCustom
open Format
open FreeSafeTyUtil

let rec get_freeovids_lp (*rev_renaming*) lp =
(*
  fstwarn "get_freeovids_lp: not implemented.%!";
  OVS.empty
*)
  let ovs = ref OVS.empty in
    for i = 0 to (get_num_rows lp) - 1 do
      let indarray, _ = get_mat_row lp i in
	ovs := Array.fold_left (fun ovs ind -> OVS.add (*(List.assoc ind rev_renaming)*) (get_ovi_from_id ind) ovs) !ovs indarray
    done;
    !ovs

type poly = P of (ovarid * float) list * float
type ineq = 
    Le of poly * float
  | Lt of poly * float
let poly_zero = P ([], 0.0)

let print_poly fmt (P(fovs, f)) =
  fprintf fmt "@[%a@]+@[%g@]" 
    (print_list
       (fun fmt (ovi, f) -> fprintf fmt "@[%g@] @[%a@]" f print_ovarid ovi)
       ~sep:"+")
    fovs
    f
let print_ineq fmt ineq =
  match ineq with
      Le(p, f) -> fprintf fmt "@[%a@] <= @[%g@]" print_poly p f
    | Lt(p, f) -> fprintf fmt "@[%a@] < @[%g@]" print_poly p f
	
(*
  let rec rename_ineq renaming ineq =
  match ineq with
  | Le(p, f) -> Le(rename_poly renaming p, f)
  | Lt(p, f) -> Lt(rename_poly renaming p, f)
  and rename_poly renaming (P(fovs, f)) =
  let fovs' =
  List.map
  (fun (ovi, f) ->
  let ovi' = try List.assoc ovi renaming with Not_found -> ovi in
  assert(ovi'.cont = None);
  (ovi', f))
  fovs
  in
  P(fovs', f)
*)

let compare_unitterm (ov1, _) (ov2, _) = compare ov1 ov2

let scalar_poly f (P(ps, c)) =
  P(List.rev_map (fun (ov, f') -> ov, (f *. f')) ps, c *. f)
let rec add_poly (P(ps1, c1)) (P(ps2, c2)) =
  let ps1, ps2 = List.sort compare_unitterm ps1, List.sort compare_unitterm ps2 in
  let rec helper ps1 ps2 =
    match ps1, ps2 with
      | [], p | p, [] -> p
      | (x1, f1)::tl1, (x2, f2)::tl2 ->
	  let cmp = compare x1 x2 in
	    if cmp = 0 then (x1, f1 +. f2)::(helper tl1 tl2)
	    else if cmp < 0 then (x1, f1)::(helper tl1 ps2)
	    else (x2, f2)::(helper ps1 tl2)
  in
    P(helper ps1 ps2, c1 +. c2)
      
let rec polymize_owner o =
  let o = find_owner o in
    match o with
      | OVar ovid -> 
	  assert(ovid.cont = None);
	  P([ovid, 1.0], 0.0)
      | OFrac f -> P([], f)
      | OScalar(f, o) ->
	  let P(ps, c) = polymize_owner o in
	  let ps = List.rev_map (fun (ov, f') -> ov, (f *. f')) ps in
	    P(ps, c)
	      (* | OPlus(o1, o2) ->
		 let p1 = polymize_owner o1 in
		 let p2 = polymize_owner o2 in
		 add_poly p1 p2 *)
      | OPlus os ->
	  List.fold_left (fun p o -> add_poly (polymize_owner o) p) poly_zero os
let n_ovid = ref 0
  (*
    let new_ovid () =
    let ret = { oid = !n_ovid } in
    incr n_ovid; ret
  *)

let rec normalize_poly (P(ps, c)) =
  let ps = List.fold_left (fun ps (ov, f) -> if f = 0.0 then ps else (ov, f)::ps) [] ps in
    P(ps, c)

module OIG = Imperative.Digraph.Concrete(
  struct 
    type t = owner
    let compare = compare_owner
    let hash = Hashtbl.hash
    let equal o1 o2 = compare_owner o1 o2 = 0
  end
)
module OIGPrint =  
  Dot(struct
	type t = OIG.t
	module V = OIG.V
	module E = OIG.E
	let iter_vertex = OIG.iter_vertex
	let iter_edges_e = OIG.iter_edges_e
	let graph_attributes g = []
	let default_vertex_attributes g = []
	let vertex_name o = 
	  fprintf str_formatter "@[\"%a\"@]" print_owner o;
	  flush_str_formatter ()
	let vertex_attributes o = []
	  (*
	    match o with 
	    | OPlus os when List.exists (fun o' -> compare_owner o o' = 0) os -> 
	    [`Width(0.01); `Height(0.01); `Style(`Filled)] 
	    | _ -> []
	  *)
	let get_subgraph o = None
	let default_edge_attributes g = []
	let edge_attributes (o1, o2) =
	  match o1, o2 with
	    | o, OPlus os when List.exists (fun o' -> compare_owner o o' = 0) os -> 
		[`Style(`Dashed); `Arrowhead(`None)] 
	    | _ -> []
      end)
    (*
      Gml.Print(OIG)
      (struct 
      let node o = 
      let s = 
      fprintf str_formatter "@[%a@]" print_owner o; 
      flush_str_formatter ()
      in
      [s, String s]
      let edge x = ["", String ""]
      end)
    *)  
let rec ineq_of_constr c =
  match c with
    | CLeOwner(o1, o2, _) ->
	let P(ps1, c1) = polymize_owner (find_owner o1) in
	let P(ps2, c2) = polymize_owner (find_owner o2) in
	let ps = add_poly (P(ps1, 0.0)) (scalar_poly (-.1.0) (P(ps2, 0.0))) in
	let c' = c2 -. c1 in
	let P(psl, c'') = normalize_poly ps in
	  if psl = [] then 
	    if c' >= c'' then None else 
	      begin
		fstdebug "<c>@[%a@]@\n" print_constr c;
		dump_relevant_instr (`Constr c);
		failwith "ineq_of_constr: Unsatisfiable polynomial."
	      end
	  else 
	    Some (Le(P(psl, 0.0), c' -. c''), [])
	      (* Some(Le(ps, c), []) *)
    | CEmpty _ | CSubtype _ | CWfTyp _ ->
	(* fstwarn "ineq_of_constr: @[%a@]@\n" print_constr c; *)
	None
    | CEqOwner _ ->
	failwith "ineq_of_constr: should not be passed"
and cs_of_mat_row (*rev_renaming*) indarray valarray pbound =
  (*
    printf "<REV_RENAMING>@\n";
    List.iter (fun (i, ov) -> printf "%d = %a@\n" i print_ovarid ov) rev_renaming;
    List.iter2 (fun ind v -> printf "ind(%d) = val(%g)@\n" ind v) (Array.to_list indarray) (Array.to_list valarray);
    printf "<PBOUND>%g@\n"pbound;
  *)
  assert(Array.length indarray = Array.length valarray);
  let os_pos, os_neg = ref [], ref [] in
    for i = 0 to (Array.length indarray) - 1 do
      let ind = indarray.(i) in
      let ov = (*List.assoc ind rev_renaming*) get_ovi_from_id ind in
	assert(ov.cont = None);
	let v = valarray.(i) in
	  if v < 0.0 then
	    os_neg := (OVar ov)::!os_neg
	  else if v > 0.0 then
	    os_pos := (OVar ov)::!os_pos
	  else
	    failwith "zero col"
    done;
    if pbound > 0.0 then os_neg := (OFrac pbound)::!os_neg
    else if pbound < 0.0 then os_pos := (OFrac (-.pbound))::!os_pos
    else ();
    let sort_os os = List.sort compare_owner os in
      os_pos := sort_os !os_pos; os_neg := sort_os !os_neg;
      match !os_pos, !os_neg with
	| [], [] -> CS.empty
	| [o], [] -> CS.singleton (CLeOwner(o, OFrac 0.0, !currentLoc))
	| [], [o] -> CS.singleton (CLeOwner(OFrac 0.0, o, !currentLoc))
	| _::_, [] -> CS.singleton (CLeOwner(OPlus !os_pos, OFrac 0.0, !currentLoc))
	| [], _::_ -> CS.singleton (CLeOwner(OFrac 0.0, OPlus !os_neg, !currentLoc))
	| _::_, [o] -> CS.singleton (CLeOwner(OPlus !os_pos, o, !currentLoc))
	| [o], _::_ -> CS.singleton (CLeOwner(o, OPlus !os_neg, !currentLoc))
	| _::_, _::_ -> CS.singleton (CLeOwner(OPlus !os_pos, OPlus !os_neg, !currentLoc))
and cs_of_lp (*rev_renaming*) (*pbounds*) lp =
(*
  fstwarn "cs_of_lp: not implemented.@\n%!";
  CS.empty
*)
  let cs = ref CS.empty in
    for i = 0 to (get_num_rows lp) - 1 do
      let indarray, valarray = get_mat_row lp i in
    (*
      printf "<ROW>@[%d@]@\n<PBOUND>@[%g@]@\n" i (get_row_ub lp i);
      List.iter2 (fun ind v ->
      printf "<ARRAY>@[%d@]=@[%g@]@\n" ind v)
      (Array.to_list indarray) (Array.to_list valarray);
    *)
	cs := CS.union (cs_of_mat_row (*rev_renaming*) indarray valarray (get_row_ub lp i)) !cs
    done;
    (* printf "<RESULT>@[%a@]@\n" print_cs !cs; *)
    !cs

and graph_of_cs cs =
  let g = OIG.create () in
  let rec add_edge_within_owner g o =
    match o with 
      | OPlus os -> 
	  List.iter 
	    (fun o' -> 
	       add_edge_within_owner g o';
	       OIG.add_vertex g o'; OIG.add_vertex g o;
	       OIG.add_edge g o' o) 
	    os
      | _ -> ()
  in
    CS.iter
      (function CLeOwner(o1, o2, _) -> 
	 add_edge_within_owner g o1;
	 add_edge_within_owner g o2;
	 OIG.add_vertex g o1;
	 OIG.add_vertex g o2;
	 OIG.add_edge g o1 o2
	 | _ -> failwith "graph_of_cs: not normalized cs")
      cs;
    g
      
let rec solve_safe slacs lp =
(*
  scale_problem lp;
  use_presolver lp true;
*)
  try
    simplex lp;
(*
    let col_primals = get_col_primals lp in
    let ret =
      List.fold_right
	(&&)
	(List.rev_map
	   (fun i -> col_primals.(i) > 0.0)
	   slacs)
	true
    in
*)
(*
      if ret then ret
      else
	(* printf "Some >0 constraints are unsatisfiable.@\n"; *)
	ret
*)
    true
  with
    | No_primal_feasible_solution -> 
	(* printf "There are unsatisfiable inequalities.@\n"; *)
	false

let rec minimize_row (*array*) (*pbounds*) slacs lp =
  let rec helper n =
(*
    if (get_num_rows lp) mod 100 = 0 then
      printf "LP size: %d@\n%!" (get_num_rows lp);
*)
    if n < 0 then lp
    else
      let indarray, valarray = get_mat_row lp n in
      let ub = get_row_ub lp n in
	delete_row lp n;
	if solve_safe slacs lp then begin
	  printf "Cannot be eliminated@\n%!";
	  add_rows lp 1;
	  let i = get_num_rows lp in
	    set_mat_row lp (i - 1) indarray valarray;
	    set_row_bounds lp (i - 1) Upper_bounded_var (-.infinity) ub
	end else
	  (* printf "Eliminated@\n%!"; *) ();
	helper (n - 1)
  in
    (* printf "LP size: %d@\n%!" (get_num_rows lp); *)
    helper ((get_num_rows lp) - 1)

(* Find an unsatisfiable constraint set which is a subset of lp. *)
exception UnsatFound
let rec maximize_minimize_row (*array*) (*pbounds*) slacs lp =
  let lp' = new_problem () in
    set_direction lp' Minimize;
    add_columns lp' (get_omaxid ());
    for i = 0 to ((get_omaxid ()) - 1) do
      set_obj_coef lp' i 1.0;
      set_col_bounds lp' i Upper_bounded_var 0.0 1.0
    done;
    set_message_level lp' 0;
    set_class lp' Linear_prog;
    try
      for i = 0 to (get_num_rows lp) - 1 do
	(* printf "LP size: %d@\n%!" (get_num_rows lp'); *)
	let indarray, valarray = get_mat_row lp i in
	let ub = get_row_ub lp i in
	  add_rows lp' 1;
	  set_mat_row lp' i indarray valarray;
	  set_row_bounds lp' i Upper_bounded_var (-.infinity) ub;
	  try
	    if solve_safe slacs lp' then () else raise UnsatFound
	  with
	    | No_dual_feasible_solution -> ()
      done;
      minimize_row slacs lp'
    with
      | UnsatFound -> lp'
	  
(*
  let rec helper n =
  if (get_num_rows lp) mod 1000 = 0 then
  printf "LP size: %d@\n%!" (get_num_rows lp);
  if n < 0 then lp
  else
  let indarray, valarray = get_mat_row lp n in
  let ub = get_row_ub lp n in
  delete_row lp n;
  if solve_safe slacs lp then begin
  printf "Cannot be eliminated@\n%!";
  add_rows lp 1;
  let i = get_num_rows lp in
  set_mat_row lp (i - 1) indarray valarray;
  set_row_bounds lp (i - 1) Upper_bounded_var (-.infinity) ub
  end else
(* printf "Eliminated@\n%!"; *) ();
  helper (n - 1)
  in
  printf "LP size: %d@\n%!" (get_num_rows lp);
  helper ((get_num_rows lp) - 1)
*)

let rec minimize_row_random (*array*) (*pbounds*) slacs lp =
  let cnt_delete = ref 0 in
  let rec delete () =
    incr cnt_delete;
    let rec loop deleted i =
      (* printf "%d@\n" (get_num_rows lp); print_flush (); *)
      if i >= get_num_rows lp then
	deleted
      else if Random.bool () then
	(* deleted *)
	let indarray, valarray = get_mat_row lp i in
	let bound = get_row_ub lp i in
	  delete_row lp i;
	  loop ((indarray, valarray, bound)::deleted) i
      else
	loop deleted (i + 1)
    in
      loop [] 0
  in
  let rec add ivs =
    (* printf "%d@\n" (get_num_rows lp); print_flush (); *)
    let added, kept = split_random ivs in
      List.iter
	(fun (inds, vals, bound) ->
	   add_rows lp 1;
	   let i = get_num_rows lp in
	     set_mat_row lp (i - 1) inds vals;
	     set_row_bounds lp (i - 1) Upper_bounded_var (-.infinity) bound)
	added;
      kept
  in
  let rec loop terminate =
    printf "LP size: %d@\n%!" (get_num_rows lp);      
    let rec add_loop deleted =
      if solve_safe slacs lp then
	let kept = add deleted in
	  printf "added: LP size: %d@\n%!" (get_num_rows lp);      
	  add_loop kept
      else
	()
    in
    let deleted = delete () in
      printf "deleted: LP size: %d@\n%!" (get_num_rows lp);      
      if terminate lp then
	lp
      else if solve_safe slacs lp then
	(add_loop deleted;
	 loop terminate)
      else
	loop terminate
  in
    (* let init_n = get_num_rows lp in *)
  let lp = loop (fun lp -> !cnt_delete >= 10) in
    lp

let rec solve_lp (*arrays*) (*pbounds*) slacs lp =
  (* let _ = fstdebug "Scaling problem@\n%!" in *)
    scale_problem lp;
  (* let _ = fstdebug "Presolver@\n%!" in *)
    let _ = use_presolver lp true in
    let _ = fstdebug "Solve!: size of lp: %d@\n%!" (get_num_rows lp) in
      if solve_safe slacs lp then
	(printf "Well-typed.@\n%!";
	 lp, true)
      else
	(printf "Ill-typed.@\n%!";
	 if !FreeSafeTyFlag.outputLinearProg && !FreeSafeTyFlag.minimizeConstr then
	   (printf "Minimized constraints are output to output.cplex.@\n%!";
	    if !FreeSafeTyFlag.randomMinimize then
	      (ignore(minimize_row_random (*pbounds*) slacs lp); minimize_row (*pbounds*) slacs lp, false)
	    else if !FreeSafeTyFlag.maximizeMinimize then
	      maximize_minimize_row (*pbounds*) slacs lp, false
	    else
	      minimize_row slacs lp, false)
	 else
	   lp, false)

let rec f topEnv (cs : constr_set) =
  (* let _ = printf "<BEFORE NORMALIZE CS>@[%a@]@\n" print_cs cs in *)
  let _ = fstdebug "Doing reduce: maxovids: %d.@\n%!" (get_omaxid ()) in
  let cs = normalize_cs cs in
    (* let _ = printf "<AFRTER NORMALIZE CS>@[%a@]@\n" print_cs cs in *)
(*
  let _ =
    if !FreeSafeTyFlag.debug then
      fstdebug "<topEnv:beforeReduction>@\n@[%a@]@\n" 
	print_tenv ((* normalize_tenv *) topEnv)
  in
*)
  let ineqs_r, slacs_r = ref [], ref [] in
    
  (* let _ = fstdebug "Making ineqs@\n%!" in *)
  let _ = 
    CS.iter 
      (fun c -> 
	 match ineq_of_constr c with
	   | Some(ineq, slacs) ->
	       ineqs_r := ineq::!ineqs_r;
	       slacs_r := slacs @ !slacs_r
	   | None -> ()) 
      cs 
  in
  (* let _ = fstdebug "Making renamings@\n%!" in *)
    (*
      let ovids = List.fold_left (fun ovs ovid -> OVS.add ovid ovs) (get_freeovids_cs cs) !slacs_r in
      if OVS.cardinal ovids (*get_omaxid ()*) = 0 (*-1*) || !ineqs_r = [] then
      ()
      else
    *)
    (*
      let ovids = get_glob_ovs () in
      let renaming =
      let cnt = ref 1 in
      List.map (fun ovid -> let ret = ovid, !cnt in incr cnt; ret) (OVS.elements ovids)
      in
    *)
  let zcoefs =
    let z = Array.make (*(List.length renaming)*) (get_omaxid ()) 1.0 in
      (*
	List.iter
	(fun ovi ->
	let i = List.assoc ovi renaming (*ovi.id*) in
	z.(i) <- 1.0)
	(List.map fst renaming);
      *)
      z
  in
  let ia, ja, ar, pbounds =
    let ia, ja, ar, pbounds = ref [], ref [], ref [], ref [] in
      (* let nowindex = ref 0 in *)
    let _ =
      List.fold_left
	(fun nowrow ineq -> 
	   match ineq with Le(P(ps, 0.0), c) ->
	     pbounds := (-.infinity, c)::!pbounds;
	     List.iter (fun (ovi, f) -> 
			  assert(ovi.cont = None);
			  if f = 0.0 then
			    ()
			  else begin
			    ia := nowrow::!ia;
			    ja := (*(List.assoc ovi renaming)*)ovi.id::!ja;
			    ar := f::!ar
			  end)
	       ps;
	     nowrow + 1
	     | _ -> failwith "creating constraint matrix: ineq is not normalized.")
	1
	!ineqs_r
    in
    let ia, ja, ar, pbounds = 
      Array.of_list (List.rev !ia), Array.of_list (List.rev !ja), Array.of_list (List.rev !ar), Array.of_list (List.rev !pbounds) in
      ia, ja, ar, pbounds
  in
    if Array.length pbounds = 0 then
      true
    else
      let xbounds = Array.make (get_omaxid ()) (*(List.length renaming)*) (0.0, 1.0) in
	(*let rev_renaming = List.map (fun (x, y) -> (y, x)) renaming in*)
      (* let _ = fstdebug "Making problems@\n%!" in *)
      let lp = new_problem () in
	set_direction lp Minimize;
	add_rows lp (Array.length pbounds);
	for i = 0 to ((Array.length pbounds) - 1) do
	  match pbounds.(i) with
            | lb, ub when lb = -.infinity && ub = infinity -> set_row_bounds lp i Free_var 0. 0.
            | lb, ub when ub = infinity -> set_row_bounds lp i Lower_bounded_var lb 0.
            | lb, ub when lb = -.infinity -> set_row_bounds lp i Upper_bounded_var 0. ub
            | lb, ub when lb = ub -> set_row_bounds lp i Fixed_var lb ub
            | lb, ub -> set_row_bounds lp i Double_bounded_var lb ub
	done;
	add_columns lp (Array.length xbounds);
	for i = 0 to ((Array.length xbounds) - 1) do
	  set_obj_coef lp i zcoefs.(i);
	  match xbounds.(i) with
            | lb, ub when lb = -.infinity && ub = infinity -> set_col_bounds lp i Free_var 0. 0.
            | lb, ub when ub = infinity -> set_col_bounds lp i Lower_bounded_var lb 0.
            | lb, ub when lb = -.infinity -> set_col_bounds lp i Upper_bounded_var 0. ub
            | lb, ub when lb = ub -> set_col_bounds lp i Fixed_var lb ub
            | lb, ub -> set_col_bounds lp i Double_bounded_var lb ub
	done;
	load_matrix_raw lp (Array.length ia) ia ja ar;
	set_message_level lp 0;
	set_class lp Linear_prog;
	(*
	  List.iter 
	  (fun (n, ovi) -> 
	  assert(ovi.cont = None);
	  set_col_name lp (n - 1) (sprintf "'o[%d]" ovi.id))
	  rev_renaming;
	*)
	let slacs = List.map (fun ovid -> assert(ovid.cont = None); (*List.assoc ovid renaming*)ovid.id) !slacs_r in
	let lp, b = solve_lp (*constrs*) (*pbounds*) slacs lp in
	  if !FreeSafeTyFlag.outputLinearProg then begin
	    write_cplex lp "output.cplex";
	    if !FreeSafeTyFlag.debug && (not !FreeSafeTyFlag.minimizeConstr) then
	      dump_instr_hash ();
	    if !FreeSafeTyFlag.minimizeConstr then begin
	      dump_relevant_instr (`Ovs (get_freeovids_lp (*rev_renaming*) lp));
	      (*
		let _ = List.iter (fun (i, ov) -> printf "%d=%a@\n" i print_ovarid ov) rev_renaming in
	      *)
	      let cs = cs_of_lp (*rev_renaming*) (*pbounds*) lp in
	      let g = graph_of_cs cs in
	      let oc = open_out "output.dot" in
	      let _ = OIGPrint.fprint_graph (formatter_of_out_channel oc) g in
		close_out oc
	    end
	  end;
	  let _ = 
	    (*
	      for i = 1 to (List.length rev_renaming) do
	      let f = get_col_primal lp (i - 1) in
	      fst (union_owner (!Cil.currentLoc) (List.assoc i rev_renaming) (OFrac f))
	      done 
	    *)
	    if b then
	      glob_ovs_iter 
		(fun ov ->
 		   let f = get_col_primal lp (ov.id - 1) in
		     if ov.cont = None then
		       fst (union_owner (!Cil.currentLoc) ov (*(List.assoc i rev_renaming)*) (OFrac f))
		     else
		       ())
		(* (get_glob_ovs ()) *)
	  in
	  let _ =
	    if !FreeSafeTyFlag.printInferredTypes && b then
	      fstdebug "<inferred types>@\n@[%a@]@\n" print_tenv (normalize_tenv !currentLoc topEnv)
	  in
	  let _ =
	    if !FreeSafeTyFlag.debug && !FreeSafeTyFlag.minimizeConstr then
	      (fstdebug "<DEBUG OUTPUT INSTR>@\n"; dump_instr_hash ());
	    ()
	  in
	    b
