
#include <stdio.h>

#include <caml/alloc.h>
#include <caml/callback.h>
#include <caml/custom.h>
#include <caml/fail.h>
#include <caml/memory.h>
#include <caml/misc.h>
#include <caml/mlvalues.h>
#include <caml/signals.h>

#include <assert.h>

#include <glpk/glpk.h>

#define Lpx_val(v) (*((LPX**)Data_custom_val(v)))

CAMLprim void ocaml_glpk_delete_row(value blp, value n)
{
  CAMLparam2(blp, n);
  int num[1+1];
  LPX *lp = Lpx_val(blp);
  num[1] = Int_val(n)+1;
  lpx_del_rows(lp, 1, num);
  /* return Val_unit; */
  CAMLreturn0;
}

CAMLprim value ocaml_glpk_get_row_ub(value blp, value n)
{
  CAMLparam2(blp, n);
  int cn = Int_val(n);
  LPX *lp = Lpx_val(blp);
  double ret = lpx_get_row_ub(lp, cn + 1);
  /* return Val_unit; */
  CAMLreturn(caml_copy_double(ret));
}

CAMLprim value ocaml_glpk_get_mat_row(value blp, value index)
{
  CAMLparam2(blp, index);
  CAMLlocal3(caml_ind, caml_val, ret);
  LPX *lp = Lpx_val(blp);
  int cindex = Int_val(index)+1;
  int rowsize;
  int *ind;
  double *val;
  unsigned int i;

  rowsize = lpx_get_mat_row(lp, cindex, NULL, NULL);
  ind = (int *)malloc(sizeof(int) * (rowsize + 1));
  val = (double *)malloc(sizeof(double) * (rowsize + 1));
  if (ind == '\0' || val == '\0') {
    assert(0);
  }
  lpx_get_mat_row(lp, cindex, ind, val);
  caml_ind = caml_alloc(rowsize, 0);
  for (i = 0; i < rowsize; ++i) {
    Store_field(caml_ind, i, Val_int(ind[i+1]));
  }
  caml_val = caml_alloc(rowsize * 2, Double_array_tag);
  for (i = 0; i < rowsize; ++i) {
    Store_double_field(caml_val, i, val[i+1]);
  }
  free(ind);
  free(val);
  ret = caml_alloc_tuple(2);
  Store_field(ret, 0, caml_ind);
  Store_field(ret, 1, caml_val);
  CAMLreturn(ret);
}

/* CAMLprim value ocaml_glpk_set_mat_row(value blp, value index, value row) */
CAMLprim void ocaml_glpk_set_mat_row(value blp, value index, 
				      value indarray, value valarray)
{
  CAMLparam4(blp, index, indarray, valarray);
  LPX *lp = Lpx_val(blp);
  /* row is an array of double value */
  unsigned int indarray_size = Wosize_val(indarray);
  unsigned int valarray_size = Wosize_val(valarray) / 2;
  unsigned int i;
  double *val;
  unsigned int *ind;
  double tmp;
  unsigned int current_index = 0;
  assert(indarray_size == valarray_size);
  ind = (unsigned int*)malloc((indarray_size + 1)* sizeof(unsigned int));
  val = (double*)malloc((valarray_size + 1) * sizeof(double));
  if (ind == NULL || val == NULL) {
    caml_failwith("Cannot allocate memory.");
  }
  for (i = 1; i <= indarray_size; ++i) {
    ind[i] = Int_val(Field(indarray, i-1));
    val[i] = Double_field(valarray, i-1);
    /* printf("ind[%d]=%d, val[%d]=%g\n", i, Int_val(Field(indarray, i-1)), i, Double_field(valarray, i-1)); */
  }
  free(ind);
  free(val);

  {
    unsigned int passed_index = Int_val(index)+1;
    /* printf("calling set_mat_row with index = %d\n", passed_index); */
    lpx_set_mat_row(lp, passed_index, indarray_size, ind, val);
  }

  CAMLreturn0;
}  

  /*
  if (i_dim <= 0)
    return Val_unit;

  j_dim = Wosize_val(Field(matrix, 0)) / 2;
  ia = (int*)malloc((i_dim * j_dim + 1) * sizeof(int));
  ja = (int*)malloc((i_dim * j_dim + 1) * sizeof(int));
  ar = (double*)malloc((i_dim * j_dim + 1) * sizeof(double));
  n = 1;

  for(i = 0; i < i_dim; i++)
  {
    assert(Wosize_val(Field(matrix, i)) == j_dim * 2);
    for(j = 0; j < j_dim; j++)
    {
      x = Double_field(Field(matrix, i), j);
      if (x != 0)
      {
        ia[n] = i + 1;
        ja[n] = j + 1;
        ar[n] = x;
        n++;
      }
    }
  }
  lpx_load_matrix(lp, n - 1, ia, ja, ar);

  free(ia);
  free(ja);
  free(ar);

  return Val_unit;
  */

CAMLprim value ocaml_glpk_load_matrix_raw(value blp, value vn, value via, value vja, value var) {
  CAMLparam5(blp, vn, via, vja, var);
  LPX *lp = Lpx_val(blp);
  int i, n;
  int *ia, *ja;
  double *ar;

  n = Int_val(vn);
  if (n <= 0) 
    return Val_unit;
  
  assert(Wosize_val(via) == n);
  assert(Wosize_val(vja) == n);
  /* 32bit use   assert(Wosize_val(var)/2 == n); */
  assert(Wosize_val(var)/2 == n);
  n = n + 1;

  ia = (int*)malloc(n * sizeof(int));
  ja = (int*)malloc(n * sizeof(int));
  ar = (double*)malloc(n * sizeof(double));

  for (i = 1; i < n; i++)
    {
      ia[i] = Int_val(Field(via, i-1));
      ja[i] = Int_val(Field(vja, i-1));
      ar[i] = Double_field(var, i-1);
    }

  lpx_load_matrix(lp, n-1, ia, ja, ar);
  free(ia);
  free(ja);
  free(ar);

  CAMLreturn0;
}

