
open Cil
(* open Utils *)
open FreeSafeTyUtil
open Pretty
(* open ContainsLock *)
open FreeSafeTyFlag

let dumpId = ref 0

let dump f name =
  if !dumpIntermFile then
    let _ = incr dumpId in
    let filename = name in
    let fout = open_out ("fstDebug_" ^ filename) in
      dumpFile defaultCilPrinter fout filename f;
      close_out fout

(*
let rec sort_globals_in_file f =
  f.globals <- sort_globals f.globals
and sort_globals gs =
  List.sort compare_global gs
and compare_global g1 g2 =
  match g1, g2 with
    | (GType _ | GCompTag _ | GCompTagDecl _ | GEnumTag _ | GEnumTagDecl _), 
      (GType _ | GCompTag _ | GCompTagDecl _ | GEnumTag _ | GEnumTagDecl _) -> 0
    | (GVarDecl _ | GVar _ | GFun _ | GAsm _ | GPragma _ | GText _), 
	(GVarDecl _ | GVar _ | GFun _ | GAsm _ | GPragma _ | GText _) -> 0 
    | (GType _ | GCompTag _ | GCompTagDecl _ | GEnumTag _ | GEnumTagDecl _), _ -> -1
    | _, (GType _ | GCompTag _ | GCompTagDecl _ | GEnumTag _ | GEnumTagDecl _) -> 1
*)

let feature : featureDescr = 
  { fd_name = "tycoon";
    fd_enabled = ref true;
    fd_description = "Perform memory usage analysis";
    fd_extraopt = [];
    fd_doit = 
      (function (f: file) ->
	 lineDirectiveStyle := Some(LineComment);
	 let _ = FreeSafeTyFlag.currentFileName := f.fileName in
	 let _ = Format.printf "Inspecting %s...@\n" f.fileName in
	 let _ = FreeSafeTyFlag.set_fileid ((Unix.stat f.fileName).Unix.st_ino) in
	 let tuneCallInstrVisitor = new TuneFunctionType.tuneCallInstrVisitor in
	 let tuneFunctionTypeVisitor = new TuneFunctionType.tuneFunctionTypeVisitor in
	 let pointifyStructVisitor = new TuneStructType.pointifyStructFields in
	 let normalizeLvalVisitor = new NormalizeLval.normalizeLvalVisitor in
	   visitCilFile tuneCallInstrVisitor f;
	   dump f "callTuned";
	   visitCilFile tuneFunctionTypeVisitor f;
	   dump f "funtypTuned";
	   visitCilFile pointifyStructVisitor f;
	   dump f "structPointified";
	   visitCilFile normalizeLvalVisitor f;
	   dump f "lvalNormalized";
	   Cfg.computeFileCFG f;
(*
	   let _ = ContainsLock.main f in
	   let setupEnvVisitor = new SetupEnv.setupEnvVisitor in
	   let _ = visitCilFile (setupEnvVisitor :> cilVisitor) f in
*)
	   let fstyfile = get_fstyfile_name f.fileName in
	   let chan = open_out_bin fstyfile in
(*
	   let constraintVisitor = new DeadlockConstr.constraintGenerationVisitor chan in
	   let _ = measure_and_print "generate" (fun () -> visitCilFile (constraintVisitor :> cilVisitor) f) () in
	   let _ =
	     try
	       measure_and_print "reduce"
		 (fun () -> constraintVisitor#dump ())
		 ();
	       Format.printf "No error found.@\n"
	     with
	       | _  -> ()
	   in
*)
	     close_out chan;
	     ()
      );
    fd_post_check = true
  }
