
open FreeSafeTyUtil
open Cil
open Int64

class computeOffsetofVisitor : cilVisitor = object(self)
  inherit nopCilVisitor
  method vexpr e = match e with
    | AddrOf(Mem(CastE(TPtr(typ, _), Const(CInt64(zero, _, _)))), ofs) ->
	let ofs_in_bit, _ = bitsOffset typ ofs in
	let _ = assert(ofs_in_bit mod 8 = 0) in
	let ofs_in_byte = ofs_in_bit / 8 in
	  ChangeTo(Const(CInt64(of_int ofs_in_byte, IUInt, None)))
    | _ -> DoChildren
end

class pointifyStructFields : cilVisitor = object(self)
  inherit nopCilVisitor

  method private pointifyCompInfo ci =
    List.iter
      (fun f ->
	  match f.fbitfield with
	    | None -> 
		begin
		  match f.ftype with
		    (* | TPtr(typ, _) when isFunctionType typ -> () *)
		    | _ -> f.ftype <- TPtr(visitCilType (self :> cilVisitor) f.ftype, [])
		end
	    | Some _ -> ())
      ci.cfields

  method vglob g = match g with
    | GCompTag(ci, loc) -> 
	let _ =
	  if ci.cname = "kretprobe_instance" then
	    ()
	in
	  self#pointifyCompInfo ci;
	  (* ChangeTo([GCompTag(self#pointifyCompInfo ci, loc)]) *)
	  SkipChildren
    | GFun(fd, loc) -> 
	let _ =
	  if fd.svar.vname = "obsolete_checksetup" then
	    ()
	in
	  DoChildren
    | GVar(vi, { init = Some CompoundInit(typ, ois) }, loc) ->
	let typ' = visitCilType (self :> cilVisitor) typ in
	let accum_gs = ref [] in
	let ois' =
	  List.fold_right
	    (fun (ofs, init) ois ->
	      match ofs with 
		| Field(f, NoOffset) ->
		    let vi' = makeGlobalVar ("__vericon_tunestruct_" ^ f.fname ^ (string_of_int (new_int ()))) f.ftype in
		      vi'.vstorage <- Static;
		      accum_gs := (GVar(vi', { init = Some init }, loc))::!accum_gs;
		      (ofs, SingleInit(AddrOf(Var vi', NoOffset)))::ois
		| _ ->
		    (ofs, init)::ois)
	    ois
	    []
	in
	  ChangeTo(!accum_gs @ [GVar(vi, { init = Some (CompoundInit(typ', ois')) }, loc)])
    | _ -> DoChildren

  method vinst i = match i with
    | Set((Var x, NoOffset), e, loc) ->
	begin match unrollType x.vtype with
	  | TComp(ci, _) ->
	      Printf.printf "WARNING!: assignment to comp type %s\n" ci.cname;
	      DoChildren
	  | _ -> DoChildren
	end
    | _ -> DoChildren

  method vtype t = match t with
    | TVoid _ | TInt _ | TFloat _ | TNamed _ 
    | TEnum _ | TBuiltin_va_list _ -> SkipChildren
    | TPtr _ | TArray _ | TFun _ -> DoChildren
(*
    | TArray(typ, eopt, loc) ->
	let eopt' = 
	  match eopt with None -> None
	    | Some e -> Some (visitCilExpr (self :> cilVisitor) e)
	in
	  ChangeTo(TArray(TPtr(visitCilType (self :> cilVisitor) typ, []), eopt', loc))
*)
    | TComp(ci, attr) -> 
	(* ChangeTo(TComp(self#pointifyCompInfo ci, attr)) *)
	SkipChildren

  method private processLval host (trackedOffsets : offset -> offset) restOffset =
    match restOffset with
      | NoOffset -> (host, trackedOffsets restOffset)
      | Index(e, ofs) ->
	  let e' = visitCilExpr (self :> cilVisitor) e in
	    self#processLval host (fun ofs' -> trackedOffsets (Index(e', ofs'))) ofs
(*
      | Index(e, ofs) ->
	  let e' = visitCilExpr (self :> cilVisitor) e in
	    (* self#processLval host (fun ofs' -> trackedOffsets (Index(e', ofs'))) ofs *)
	  let h' = Mem(Lval(host, trackedOffsets (Index(e', NoOffset)))) in
	    self#processLval h' (fun ofs -> ofs) ofs
*)
      | Field(f, ofs) when f.fbitfield = None ->
	  begin
	    match f.ftype with
(*
	      | TPtr(typ, _) when isFunctionType typ ->
		  self#processLval host (fun ofs' -> trackedOffsets (Field(f, ofs'))) ofs
*)
	      | _ ->
		  let h' = Mem(Lval(host, trackedOffsets (Field(f, NoOffset)))) in
		    self#processLval h' (fun ofs -> ofs) ofs
		      
	  end
      | Field(f, NoOffset) ->
	  (host, trackedOffsets restOffset)
      | _ -> failwith "processLval: cannot occur"

  method vlval (h, ofs) =
    let _ =
      if !currentLoc.line = 166 then
	()
    in
    let h =
      match h with Var x -> Var x | Mem(e) -> Mem(visitCilExpr (self :> cilVisitor) e)
    in
      ChangeTo(self#processLval h (fun ofs -> ofs) ofs)
      
  method vexpr e = match e with
    | AddrOf(lv) ->
	let lv' = visitCilLval (self :> cilVisitor) lv in
	  begin match lv' with
	    | Mem(e), NoOffset -> ChangeTo(e)
	    | _ -> ChangeTo(AddrOf(lv'))
	  end
    | _ -> DoChildren
end
